/*******************************************************************************

  ioaccess.c: IO access code for Lanner Platfomr Watchdog/Bypass program

  Lanner Platform Miscellaneous Utility
  Copyright(c) 2012 Lanner Electronics Inc.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer,
     without modification.
  2. Redistributions in binary form must reproduce at minimum a disclaimer
     similar to the "NO WARRANTY" disclaimer below ("Disclaimer") and any
     redistribution must be conditioned upon including a substantially
     similar Disclaimer requirement for further binary redistribution.
  3. Neither the names of the above-listed copyright holders nor the names
     of any contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  Alternatively, this software may be distributed under the terms of the
  GNU General Public License ("GPL") version 2 as published by the Free
  Software Foundation.

  NO WARRANTY
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTIBILITY
  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  THE POSSIBILITY OF SUCH DAMAGES.

*******************************************************************************/



#include "../include/config.h"


#ifdef DJGPP

/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
/* For DOS DJGPP */
#include <dos.h>
#include <inlines/pc.h>

#else //DJGPP
/* For Linux */


#ifdef DIRECT_IO_ACCESS
/* For Linux direct io access code */
/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if defined(LINUX_ENV)
#include <sys/io.h>
#endif

#if defined(FreeBSD_ENV)
#include <machine/cpufunc.h>
#endif


#include <time.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#define delay(x) usleep(x)
#endif


#ifdef MODULE

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/io.h>
#include <linux/delay.h>

#undef delay
#define delay(x) mdelay(x)
#undef fprintf
#define fprintf(S, A)  printk(A)

#endif //MODULE

#ifdef KLD_MODULE

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/errno.h>

#include <machine/bus.h>
#include <machine/resource.h>

#endif

#endif

/* local include file */
#include "../include/ioaccess.h"

#if (defined(MODULE) || defined(DIRECT_IO_ACCESS) || defined(KLD_MODULE))


/*
 * Platform Depend GPIOs Interface for Watchdog and Lan bypass
 */
/*
 *------------------------------------------------------------------------------
 * LEB-7050 Version V1.0
 *
 * LEB-7050 embedded with Watchdog timer functions.
 *
 * Refer to Fintek 81865 datasheet for details.
 * The truth table is defined as below:
 *
 * Below setting is to determine system behavior while watchdog timer expired.
 *
 *------------------------------------------------------------------------------




 *------------------------------------------------------------------------------
 */
/*
 * Device Depend Definition : Fintek 81865
 */

#define BANK_REG	0x07
#define LDN7		0x07

#define INDEX_PORT	0x4e
#define DATA_PORT	0x4f

#define	CR20_REG	0x20	//This register indicate the device's ID
#define f81865_ID	0x07	//;Fintek


//;********************* Enter SIO *****************************************************
void Enter_Configuation(void)
{
    	outportb(INDEX_PORT, 0x87);  
        delay(1);
    	outportb(INDEX_PORT, 0x87); 
	return;
}

//;********************* Exit SIO *****************************************************
void Exit_Configuation(void)
{
    	outportb(INDEX_PORT, 0xaa);  						//Exit Fintek 81865 Configuration
	return;
}

//;********************* Read SIO *****************************************************
unsigned char read_f81865_reg(int LDN, int reg)
{
        unsigned char tmp;	
	Enter_Configuation();		
        outportb(INDEX_PORT, BANK_REG); //LDN register
        delay(1);
        outportb(DATA_PORT, LDN);
        delay(1);
        outportb(INDEX_PORT, reg);
        delay(1);       
        tmp = inportb(DATA_PORT);
	Exit_Configuation();         
	return tmp;
}

//;********************* Write SIO *****************************************************
void write_f81865_reg(int LDN, int reg, int value)
{
	Enter_Configuation();		
        outportb(INDEX_PORT, BANK_REG); //LDN register
        delay(1);
        outportb(DATA_PORT, LDN);
        delay(1);
        outportb(INDEX_PORT, reg);
        delay(1);
        outportb(DATA_PORT, value);
	Exit_Configuation();        
        return;
}

void start_watchdog_timer(int watchdog_time)
{
	write_f81865_reg(LDN7, 0x30, 0x01);							//;enable LDN7
	write_f81865_reg(LDN7, 0xf6, watchdog_time); 		//LDN=7, CRF6, set watchdog timer time-out value	
	write_f81865_reg(LDN7, 0xf5, read_f81865_reg(LDN7, 0xF5) | 0x20);		//;//;enable WD	
	delay(20);	
	return;	
}

void stop_watchdog_timer(void)
{

	/* stop timer */
	write_f81865_reg(LDN7, 0xf5, read_f81865_reg(LDN7, 0xF5) & ~0x20);		//;//;disable WD		
	delay(20);	
	write_f81865_reg(LDN7, 0xf6, 0);
}

int wd_gpio_init(void)
{		
	int ret=0;
	write_f81865_reg(LDN7, 0x30, 0x01);		//;enable LDN7 		
	write_f81865_reg(LDN7, 0xf5, read_f81865_reg(LDN7, 0xF5) | 0x51);		//; clean status, set second 
	delay(20);	
	write_f81865_reg(LDN7, 0xf5, read_f81865_reg(LDN7, 0xF5) & ~0x20);		//;disable WD 		
	delay(20);	
//cloud-test	write_f81865_reg(LDN7, 0xf6, 0x00); 	//;LDN=7, CRF6, stop timer first	
//cloud-test	write_f81865_reg(LDN7, 0xfa, 0x01);		//;clear status bit, PME Enable Configuration index
	write_f81865_reg(LDN7, 0xfa, 0x11);		//cloud-test	//;clear status bit, PME Enable Configuration index
	return ret;

}

void set_bypass_enable_when_system_off(unsigned long pair_no)
{
	return;
}


void set_bypass_disable_when_system_off(unsigned long pair_no)
{
	return;
}

void set_runtime_bypass_enable(unsigned long pair_no)
{
	return;
}

void set_runtime_bypass_disable(unsigned long pair_no)
{
	return;
}

void set_wdto_state_system_reset(void)
{
	return;
}

void set_wdto_state_system_bypass(void)
{
	return;
}

#endif
